#include "7zipDataStream.h"

STDMETHODIMP CInDataStream::Read(void *data, UInt32 size, UInt32 *processedSize)
{
	if ( processedSize != NULL )
		*processedSize = 0;

	if ( m_lCurPos >= m_lMaxSize )
		return S_OK;

	// check its not past the data
	long realread = size;
	if ( ((long)size + m_lCurPos) > m_lMaxSize )
		realread = (m_lMaxSize - m_lCurPos);

	memcpy ( data, (m_sData + m_lCurPos), realread );
//	memcpy ( data, m_sData, realread );

	m_lCurPos += realread;
	
	if ( processedSize != NULL )
		*processedSize = (UInt32)realread;
  
	return S_OK;
}

STDMETHODIMP CInDataStream::Seek(Int64 offset, UInt32 seekOrigin, UInt64 *newPosition)
{
	// beginning
	UInt64 newpos = 0;
	if ( seekOrigin == SEEK_SET )
		newpos = offset;
	// current position
	else if ( seekOrigin == SEEK_CUR )
		newpos = m_lCurPos + offset;
	// end position
	else if ( seekOrigin == SEEK_END )
		newpos = m_lMaxSize - offset;
	else
		return STG_E_INVALIDFUNCTION;

	if ( newpos > m_lMaxSize )
	{
		m_lCurPos = 0;
		return E_FAIL;
	}

	m_lCurPos = (long)newpos;

	if ( newPosition != NULL )
		*newPosition = (UInt64)newpos;
	
	return S_OK;
}

STDMETHODIMP CInDataStream::GetSize ( UInt64 *size )
{
	*size = m_lMaxSize;
	return S_OK;
}


/*
################################################################################################
########################                Out Data Stream                 ########################
################################################################################################
*/

void COutDataStream::Create ( long size )
{
	if ( m_sData )
		free ( m_sData );

	m_sData = (unsigned char *)malloc ( sizeof(unsigned char) * size );
	m_lMaxSize = size;
	m_lCurPos = 0;
}

STDMETHODIMP COutDataStream::Write ( const void *data, UInt32 size, UInt32 *processedSize )
{
	if ( processedSize != NULL )
		*processedSize = 0;

	if ( !m_sData )
		return E_FAIL;

	if ( m_lCurPos >= m_lMaxSize )
		return S_OK;

	long realread = size;

	if ( (m_lCurPos + (long)size) > m_lMaxSize )
	{
		long extraneeded = (m_lCurPos + (long)size) - m_lMaxSize;
		m_lMaxSize += extraneeded;
		m_sData = (unsigned char *)realloc ( m_sData, sizeof(unsigned char) * m_lMaxSize );
	}

	memcpy ( m_sData + m_lCurPos, data, size );
	m_lCurPos += realread;

	if ( processedSize != NULL )
		*processedSize = (UInt32)realread;

	return S_OK;
}

STDMETHODIMP COutDataStream::Seek(Int64 offset, UInt32 seekOrigin, UInt64 *newPosition)
{
	// beginning
	UInt64 newpos = 0;
	if ( seekOrigin == SEEK_SET )
		newpos = offset;
	// current position
	else if ( seekOrigin == SEEK_CUR )
		newpos = m_lCurPos + offset;
	// end position
	else if ( seekOrigin == SEEK_END )
		newpos = m_lMaxSize - offset;
	else
		return STG_E_INVALIDFUNCTION;

	if ( newpos > m_lMaxSize )
	{
		m_lCurPos = 0;
		return E_FAIL;
	}

	m_lCurPos = (long)newpos;

	if ( newPosition != NULL )
		*newPosition = (UInt64)newpos;
	
	return S_OK;
}

STDMETHODIMP COutDataStream::SetSize(Int64 newSize)
{
  return E_FAIL;
}
